@@ -11,7 +11,6 @@ import csv
 from glob import glob
 
 from result import Ok, Err, Result
-from collections.abc import Mapping
 
 KEY_STUDENT_ID = 'id'
 KEY_STUDENT_NAME_FIRST = 'first_name'
@@ -154,7 +153,7 @@ class Ep2Group:
 
         return students
 
-    def student_list(self):
+    def student_list(self) -> [str]:
         stud_csv = self.students_csv()
         students = []
 
@@ -237,8 +236,8 @@ class Ep2:
         # TODO check timing constraints? e.g. commit time can't be later than 5 minutes after hand-in?
         try:
             project.tags.create({'tag_name': tag, 'ref': commit_id}, sudo=self.sudo)  # create tag for commit
-        except gitlab.exceptions.GitlabCreateError:  # tag already exists, fail gracefully
-            click.secho('Tag already exists in project', fg='red')
+        except gitlab.exceptions.GitlabCreateError as err:  # tag already exists, fail gracefully
+            click.secho('Error when tagging project: {}'.format(err), fg='red')
             return False
 
         if self.verbose:
@@ -246,7 +245,7 @@ class Ep2:
 
         return True
 
-    def clone_or_update(self, repo_dir, project, tag):
+    def clone_or_update(self, repo_dir, project, tag) -> Result:
         """Clones or updates a repository into repo_dir"""
 
         try:
@@ -342,36 +341,81 @@ class Ep2:
         if os.path.exists(dst):
 
             if os.path.exists(dst_src_dir):
-                os.unlink(dst_src_dir)
+                try:
+                    os.unlink(dst_src_dir)
+                except IsADirectoryError:
+                    try:
+                        os.rmdir(dst_src_dir)
+                    except OSError:
+                        shutil.rmtree(dst_src_dir)
 
             if os.path.exists(dst_angabe_dir):
-                os.unlink(dst_angabe_dir)
+                try:
+                    os.unlink(dst_angabe_dir)
+                except IsADirectoryError:
+                    try:
+                        os.rmdir(dst_angabe_dir)
+                    except OSError:
+                        shutil.rmtree(dst_angabe_dir)
 
             if os.path.exists(dst_test_dir):
-                os.unlink(dst_test_dir)
+                try:
+                    os.unlink(dst_test_dir)
+                except IsADirectoryError:
+                    try:
+                        os.rmdir(dst_test_dir)
+                    except OSError:
+                        shutil.rmtree(dst_test_dir)
 
             pre_eval_results = glob(os.path.join(dst, 'pre_eval_*.yml'))
 
             for pre_eval_result in pre_eval_results:
-                os.unlink(pre_eval_result)
+                os.unlink(pre_eval_result)      # should work the same for single files and symlinks
 
         else:
             os.makedirs(dst, exist_ok=True)
 
-        os.symlink(src_src_dir, dst_src_dir, target_is_directory=True)
-        os.symlink(src_angabe_dir, dst_angabe_dir, target_is_directory=True)
+        try:
+            os.symlink(src_src_dir, dst_src_dir, target_is_directory=True)
+        except OSError as e:
+            click.echo('User has no rights to create symlinks! See https://stackoverflow.com/a/65504258 to set proper '
+                       'privileges under Windows 10.')
+            click.echo('System error: {}'.format(e))
+            exit(5)
+
+        try:
+            os.symlink(src_angabe_dir, dst_angabe_dir, target_is_directory=True)
+        except OSError as e:
+            click.echo('User has no rights to create symlinks! See https://stackoverflow.com/a/65504258 to set proper '
+                       'privileges under Windows 10.')
+            click.echo('System error: {}'.format(e))
+            exit(5)
 
         if copy_pre_eval and ue is not None:
             src_pre_eval = os.path.join(self.tutor_repo(), group, project, 'pre_eval_{}.yml'.format(ue))
             dst_pre_eval = os.path.join(dst, 'pre_eval_{}.yml'.format(ue))
 
-            os.symlink(src_pre_eval, dst_pre_eval)
+            try:
+                os.symlink(src_pre_eval, dst_pre_eval)
+            except OSError as e:
+                click.echo(
+                    'User has no rights to create symlinks! See https://stackoverflow.com/a/65504258 to set proper '
+                    'privileges under Windows 10.')
+                click.echo('System error: {}'.format(e))
+                exit(5)
 
         if copy_tests and ue is not None:
             src_test_dir = os.path.join(self.tutor_repo(), 'unittests', str(ue))
 
             if os.path.exists(src_test_dir):
-                os.symlink(src_test_dir, dst_test_dir, target_is_directory=True)
+                try:
+                    os.symlink(src_test_dir, dst_test_dir, target_is_directory=True)
+                except OSError as e:
+                    click.echo(
+                        'User has no rights to create symlinks! See https://stackoverflow.com/a/65504258 to set proper '
+                        'privileges under Windows 10.')
+                    click.echo('System error: {}'.format(e))
+                    exit(5)
 
     def test_info_csv(self) -> str:
         return os.path.join(self.tutor_repo(), "test_info.csv")
