@@ -155,7 +155,7 @@ def validate_grading(ctx, param, value):
         return value
     if value is None:
         return value
-    match = re.match('^[+\\-~0]*$', value)
+    match = re.match('^[+\\-~]*$', value)
     if match is not None:
         return value
     else:
@@ -210,7 +210,7 @@ def grade(ctx, group, ue: int, student, grading, student_feedback, solution_rema
     elif curr_ue_type == ExerciseType.Normal:
         if grading is None:
             grading = click.prompt('Grading')
-            while not re.match('^[+\\-~0]*$', grading):
+            while not re.match('^[+\\-~]*$', grading):
                 grading = click.prompt('Grading')
     else:
         click.secho('Unknown exercise type %s' % curr_ue_type, fg='red')
@@ -351,9 +351,107 @@ def grade_interactive(ctx, group, ue: int, idea: bool):
     group = ep2.group(group)
     f_info = FileInformation('Grade')
 
-    students = group.student_info()
-    for student in students:
-        print(student)
+    pre_eval_csv_file = group.pre_eval_csv(ue)  # file for reading
+    tmp_file = pre_eval_csv_file + '.tmp'  # file for writing
+
+    curr_ue_type = ue_type(ue)
+
+    if curr_ue_type == ExerciseType.Team:
+        click.secho('Interactive grading mode currently does not support team exercises!')
+        return
+
+    quit = False
+
+    with open(pre_eval_csv_file, 'r', encoding='utf-8-sig') as infile:
+        f_info.open_write(tmp_file, True)
+        with open(tmp_file, 'w', encoding='utf-8') as outfile:
+
+            assert curr_ue_type == ExerciseType.Normal      # should only be false, if a new exercise type is added in the future
+
+            reader = csv.DictReader(infile, fieldnames=pre_eval_csv_fieldnames(ep2, ue), strict=True, restkey=KEY_INVALID)
+
+            headers = next(reader, None)
+            if not validate_headers(headers):
+                click.secho('Malformed file: %s. Invalid headers!' % pre_eval_csv_file, fg='red')
+                f_info.print_info()
+                exit(1)
+
+            writer = csv.DictWriter(outfile, fieldnames=pre_eval_csv_fieldnames(ep2, ue), lineterminator='\n')
+
+            writer.writeheader()
+
+            for row in reader:
+                if KEY_INVALID in row:
+                    click.secho('Malformed file: %s' % pre_eval_csv_file, fg='red')
+                    f_info.print_info()
+                    exit(1)
+
+                if not check_row(row):
+                    click.secho('Malformed file: %s. Missing column(s)!' % pre_eval_csv_file, fg='red')
+                    f_info.print_info()
+                    exit(1)
+
+                student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
+
+                if student_id.is_err():
+                    click.secho(
+                        "[ERROR] unrecoverable error in student id normalization: {}".format(student_id.err()),
+                        fg='red', bold=True)
+                    return
+                else:
+                    student_id = student_id.ok()
+
+                row[KEY_STUDENT_ID] = student_id
+
+                if row[KEY_UE_GRADING] == '_' and not quit:
+                    current_uebung = ep2.local_repo(student_id)
+
+                    clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), ep2.get_ue_tag(ue))
+                    if clone_result.is_err():
+                        click.secho("Could not checkout submission for {}, {}".format(student_id, clone_result.value), fg='red')
+                    else:
+                        if idea:
+                            ep2.idea_checkout(student_id, ue=ue, group=group.name, copy_tests=True, copy_pre_eval=True)
+                            click.secho('Project %s loaded into IntelliJ Idea' % student_id, fg='green')
+
+                    subtasks = ep2.get_exercise_subtasks(ue)
+
+                    click.echo("Enter Grading for student {} ({} x [+~-])".format(student_id, len(subtasks)))
+                    click.echo()
+                    click.echo("Gradable subtasks:")
+                    for i, subtask in enumerate(subtasks):
+                        click.echo("    {} {} ({} P)".format(click.style("{}.".format(i + 1), bold=True), subtask.name, subtask.max_points))
+
+                    grading = click.prompt("Grading (q for quit)")
+                    if grading == 'q':
+                        quit = True
+                    match = re.match('^[+\\-~]*$', grading)
+
+                    while (match is None or len(grading) != len(subtasks)) and not quit:
+                        grading = click.prompt("Grading")
+                        if grading == 'q':
+                            quit = True
+                        match = re.match('^[+\\-~]*$', grading)
+
+                    if not quit:
+                        remarks = click.prompt("Remarks", default='', show_default=True)
+                        feedback = click.prompt("Feedback", default='', show_default=True)
+
+                        row[KEY_UE_GRADING] = grading
+                        row[KEY_UE_FEEDBACK] = feedback
+                        row[KEY_UE_REMARKS] = remarks
+
+                writer.writerow(row)
+                outfile.flush()
+
+    f_info.open_write(pre_eval_csv_file, True)
+    shutil.move(tmp_file, pre_eval_csv_file)  # replace old file with tmp file
+    f_info.delete(tmp_file, True)
+
+    f_info.print_info()
+    click.echo('Submit your results using ' + click.style('%s submit' % EVAL_COMMAND_NAME, bold=True) + '.')
+    click.echo(
+        'For a list of ungraded exercises run ' + click.style('%s list ungraded' % EVAL_COMMAND_NAME, bold=True) + '.')
 
 
 @cli.command()
