@@ -257,7 +257,7 @@ def submission(ctx, group, ue, late, idea, grade, single):
                 click.echo('No teams left to add, use ' + click.style('ep2_ex_text man_submission', bold=True)
                            + ' to add a submission manually')
 
-            for team in index.remaining_teams:
+            for team in sorted(index.remaining_teams, key=lambda team: team.name):
                 click.secho('Enter repository owner for team {}'.format(team.name))
 
                 for i, member in enumerate(team.students):
@@ -301,21 +301,23 @@ def submission(ctx, group, ue, late, idea, grade, single):
                 else:
                     members = [(repo_owner, 'o'), (editor, 'e')]
 
-                if idea:
-                    current_uebung = ep2.ue_repo(repo_owner)
+                if grade:
+                    current_uebung = ep2.local_repo(repo_owner)
 
-                    if not ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST_LATE % ue):
-                        if not ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST % ue):
-                            click.secho("Could not checkout submission from " + repo_owner +
-                                        ", no tagged commit found", fg='red')
+                    if ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST_LATE % ue).is_err():
+                        clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST % ue)
+                        if clone_result.is_err():
+                            click.secho("Could not checkout submission from {} for team {}, {}"
+                                        .format(student_id, row[KEY_EX_TEST_TEAM], clone_result.value), fg='red')
                         else:
+                            if idea:
+                                ep2.idea_checkout(repo_owner)
+                                click.secho('Project %s loaded into IntelliJ Idea' % repo_owner, fg='green')
+                    else:
+                        if idea:
                             ep2.idea_checkout(repo_owner)
                             click.secho('Project %s loaded into IntelliJ Idea' % repo_owner, fg='green')
-                    else:
-                        ep2.idea_checkout(repo_owner)
-                        click.secho('Project %s loaded into IntelliJ Idea' % repo_owner, fg='green')
 
-                if grade:
                     points = click.prompt("Points", type=click.IntRange(0, 4), show_choices=True)
                     remarks = click.prompt("Remarks", default='', show_default=True)
                     feedback = click.prompt("Feedback", default='', show_default=True)
@@ -338,31 +340,33 @@ def submission(ctx, group, ue, late, idea, grade, single):
 
             writer.writeheader()
 
-            action = prompt_manual('Create submissions for: (all, manual, check)', show_choices=True,
+            action = prompt_manual('Create submissions for: (all, check)', show_choices=True,
                                    show_default=True, default='a', type=click.Choice(['a', 'c']), suffix='> ')
 
             if action.is_ok():
                 action = action.ok()
 
                 if action == 'a':
-                    for student in index.students:
-                        if idea:
-                            current_uebung = ep2.ue_repo(student)
+                    for student in sorted(index.students):
+                        if grade:
+                            current_uebung = ep2.local_repo(student)
 
-                            if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
-                                                       TAG_NAME_EX_TEST_LATE % ue):
-                                if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
-                                                           TAG_NAME_EX_TEST % ue):
-                                    click.secho("Could not checkout submission from " + student +
-                                                ", no tagged commit found", fg='red')
+                            if ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                                       TAG_NAME_EX_TEST_LATE % ue).is_err():
+                                clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                                                   TAG_NAME_EX_TEST % ue)
+                                if clone_result.is_err():
+                                    click.secho("Could not checkout submission from {}, {}"
+                                                .format(student, clone_result.value), fg='red')
                                 else:
+                                    if idea:
+                                        ep2.idea_checkout(student)
+                                        click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+                            else:
+                                if idea:
                                     ep2.idea_checkout(student)
                                     click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
-                            else:
-                                ep2.idea_checkout(student)
-                                click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
 
-                        if grade:
                             points = click.prompt("Points for {}".format(student), type=click.IntRange(0, 4),
                                                   show_choices=True)
                             remarks = click.prompt("Remarks", default='', show_default=True)
@@ -380,23 +384,25 @@ def submission(ctx, group, ue, late, idea, grade, single):
 
                     for student in sorted(index.students):
                         if click.confirm('Add submission for student {}?'.format(student)):
-                            if idea:
-                                current_uebung = ep2.ue_repo(student)
+                            if grade:
+                                current_uebung = ep2.local_repo(student)
 
-                                if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
-                                                           TAG_NAME_EX_TEST_LATE % ue):
-                                    if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
-                                                               TAG_NAME_EX_TEST % ue):
-                                        click.secho("Could not checkout submission from " + student +
-                                                    ", no tagged commit found", fg='red')
+                                if ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                                           TAG_NAME_EX_TEST_LATE % ue).is_err():
+                                    clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(student),
+                                                                       TAG_NAME_EX_TEST % ue)
+                                    if clone_result.is_err():
+                                        click.secho("Could not checkout submission from {}, {}"
+                                                    .format(student, clone_result.value), fg='red')
                                     else:
+                                        if idea:
+                                            ep2.idea_checkout(student)
+                                            click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
+                                else:
+                                    if idea:
                                         ep2.idea_checkout(student)
                                         click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
-                                else:
-                                    ep2.idea_checkout(student)
-                                    click.secho('Project %s loaded into IntelliJ Idea' % student, fg='green')
 
-                            if grade:
                                 points = click.prompt("Points", type=click.IntRange(0, 4), show_choices=True)
                                 remarks = click.prompt("Remarks", default='', show_default=True)
                                 feedback = click.prompt("Feedback", default='', show_default=True)
@@ -425,7 +431,7 @@ def submission(ctx, group, ue, late, idea, grade, single):
 @click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', type=int)
 @click.option("--student", required=True, prompt=True, help='student id of the student, that should be graded',
               callback=validate_student_id)
-@click.option("--role", prompt=True, type=click.Choice(['o', 'w', 't']), help='the role of the student for this exercise test')
+@click.option("--role", prompt=True, type=click.Choice(['o', 'e', 't']), help='the role of the student for this exercise test')
 @click.option("--late/--on-time", default=False, help='if submission was late')
 @click.option("--team", prompt=True, help='the name of the team to add (this will not be verified!)')
 @click.pass_context
@@ -573,11 +579,16 @@ def checkout(ctx, group, ue: int):
                 else:
                     student_id = student_id.ok()
 
+                click.echo('Checking out repository {} for team {}'.format(student_id, row[KEY_EX_TEST_TEAM]))
+
                 current_uebung = os.path.join(uebung_path, student_id)
-                if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST_LATE % ue):
-                    if not ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST % ue):
-                        click.secho("Could not checkout submission from " + student_id +
-                                    ", no tagged commit found", fg='red')
+                if ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST_LATE % ue).is_err():
+                    clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST % ue)
+                    if clone_result.is_err():
+                        click.secho("Could not checkout submission from {} for team {}, {}".format(student_id,
+                                                                                                   row[KEY_EX_TEST_TEAM],
+                                                                                                   clone_result.value),
+                                    fg='red')
 
     click.echo('To grade the exercises run ' + click.style('%s grade' % EX_TEST_COMMAND_NAME, bold=True) + '.')
     click.echo('To load the projects into IntelliJ IDEA run ' + click.style('%s idea' % UTIL_COMMAND_NAME, bold=True) + '.')
