@@ -98,7 +98,9 @@ def util_init(semester, git_home, gitlab_url, gitlab_repo_prefix, gitlab_access_
 
     # Clone or update tutor repository
     ep2 = Ep2(verbose=True, config=config)
-    ep2.clone_or_update(ep2.tutor_repo(), os.path.join(config.get('Gitlab', 'RepoPrefix'), 'org', 'tutorinnen').replace("\\", "/"), 'master')
+    clone_result = ep2.clone_or_update(ep2.tutor_repo(), os.path.join(config.get('Gitlab', 'RepoPrefix'), 'org', 'tutorinnen').replace("\\", "/"), 'master')
+    if clone_result.is_err():
+        click.secho('error while cloning student repository: {}', clone_result.is_err())
 
     if os.path.exists(conf_file):
         f_info.add_file(conf_file, Action.MODIFY)
